

"""
      NOTE:
          in all `BaseMail` mathod there is a keyword argument called `port`, that port is the port of your SMTP server which will listen, make sure to put the port that is convenient to your SMTP server e.g for gmail is 465
          
          
      To tests your mail locally, boot up the python mail server on a different terminal by:
      $:/~    `python3 -m smtpd -c DebuggingServer -n localhost:7000`
          
      and then open another terminal and run your python script including the below code in it:
      >>>    from kyaah import BaseMail

      >>>    subj = f"Hellow world!"
      >>>    body = "Lorem ipsum dolor sit amet consectetur adipisicing elit. Rerum voluptate ipsum voluptatum doloribus, incidunt totam doloremque."


      >>>    base = BaseMail(
      ...      from_usr = "myself@mail.com",
      ...      mail_passwd = "myP@$$w)rD",
      ...      to_usr = ["userOne@mail.com", "userTwo@mail.com"],
      ...      subject = subj,
      ...      body = body,
      ...      server = "<your_mail_server>",
      ...      )
      >>>    base.local_mail()
      ------------------------------------------------------------------------------------------------------
      To send a text mail to someone:
          
      >>>    from kyaah import BaseMail

      >>>    subj = f"Hellow world!"
      >>>    body = "Lorem ipsum dolor sit amet consectetur adipisicing elit, rerum voluptate ipsum volupt."


      >>>    base = BaseMail(
      ...      from_usr = "myself@mail.com",
      ...      mail_passwd = "myP@$$w)rD",
      ...      to_usr = ["userOne@mail.com", "userTwo@mail.com"],
      ...      subject = subj,
      ...      body = body,
      ...      server = "<your_mail_server>",
      ...      )
      >>>    base.send_mail(port=465)
      ------------------------------------------------------------------------------------------------------
      To send a text mail together with an image:
      >>>    from kyaah import BaseMail

      >>>    subj = f"Hellow world!"
      >>>    body = "Lorem ipsum dolor sit amet consectetur adipisicing elit, rerum voluptate ipsum volupt."


      >>>    base = BaseMail(
      ...      from_usr = "myself@mail.com",
      ...      mail_passwd = "myP@$$w)rD",
      ...      to_usr = ["userOne@mail.com", "userTwo@mail.com"],
      ...      subject = subj,
      ...      body = body,
      ...      server = "<your_mail_server>",
      ...      )
      >>>    base.mail_with_image('image_1.jpg', port=465)
          
      If you want to send more than one image, pass a list of the images as argument into the `mail_with_image` method like:
      >>>    base.mail_with_image(['image_1.png', 'image_2.jpg'], port=465)
      ------------------------------------------------------------------------------------------------------
      To send mail together with files, pass a list of the files you want to send as an argument in the `mail_with_file` method like:
      >>>    from kyaah import BaseMail

      >>>    subj = f"Hellow world!"
      >>>    body = "Lorem ipsum dolor sit amet consectetur adipisicing elit, rerum voluptate ipsum volupt."


      >>>    base = BaseMail(
      ...      from_usr = "myself@mail.com",
      ...      mail_passwd = "myP@$$w)rD",
      ...      to_usr = ["userOne@mail.com", "userTwo@mail.com"],
      ...      subject = subj,
      ...      body = body,
      ...      server = "<your_mail_server>",
      ...      )

      >>>    base.mail_with_file(['file_1.pdf', 'file_2.docx'], port=465)
      ------------------------------------------------------------------------------------------------------
      To send mail together with html page call the `mail_with_page` method and pass a keyword args `file` with a value of the absolute location of your html file in the `mail_with_page` method, like:
      
      >>>    from kyaah import BaseMail

      >>>    subj = f"Hellow world!"
      >>>    body = "Lorem ipsum dolor sit amet consectetur adipisicing elit, rerum voluptate ipsum volupt."


      >>>    base = BaseMail(
      ...      from_usr = "myself@mail.com",
      ...      mail_passwd = "myP@$$w)rD",
      ...      to_usr = ["userOne@mail.com", "userTwo@mail.com"],
      ...      subject = subj,
      ...      body = body,
      ...      server = "<your_mail_server>",
      ...      )

      >>>    base.mail_with_page(file='index.html', port=465)
      
      If you want to test we provide you with a test feature which in short is to the keyword args of `file='default'` like:
      
      >>>    base.mail_with_page(file='default', port=465)
      ------------------------------------------------------------------------------------------------------
      
  ####   ######   ####   #    #  #####      #     #####   #   #
 #       #       #    #  #    #  #    #     #       #      # #
  ####   #####   #       #    #  #    #     #       #       #
      #  #       #       #    #  #####      #       #       #
 #    #  #       #    #  #    #  #   #      #       #       #
  ####   ######   ####    ####   #    #     #       #       #
      
      FOR SECURITY REASON
      we provide you a class called `Vault`. This class will look your email address and password/app_password in your sysytem environment variable. And you don't need to change anything in your code, all the mothods is thesame. Look more for this class in your interpreter by calling the  '''help(Vault)'''  Use this class in production instead of the  '''BaseMail'''  class. example:
      
      >>>    from kyaah import Vault

      >>>    subj = f"Hellow world!"
      >>>    body = "Lorem ipsum dolor sit amet consectetur adipisicing elit, rerum voluptate ipsum volupt."


      >>>    base = Vault(
      ...      from_usr = "myself@mail.com",
      ...      mail_passwd = "myP@$$w)rD",
      ...      to_usr = ["userOne@mail.com", "userTwo@mail.com"],
      ...      subject = subj,
      ...      body = body,
      ...      server = "<your_mail_server>",
      ...      )

      >>>    base.send_mail(port=465)
      ------------------------------------------------------------------------------------------------------
      
  #####   ####   #    #  ######  #    #   ####
    #    #    #  #   #   #       ##   #  #
    #    #    #  ####    #####   # #  #   ####
    #    #    #  #  #    #       #  # #       #
    #    #    #  #   #   #       #   ##  #    #
    #     ####   #    #  ######  #    #   ####
    
      There is a OTP code feature if you want to send an OTP code for verification by:
      >>>  from kyaah import Tokens
      
      >>>  otp_code = Tokens.mail_otp()
      636937
      
      You can also specify the length of numbers you want by passing a key argument of `_range` in the method like:
      >>>  otp_code = Tokens.mail_otp(_range = 10)
      5373593635
      ------------------------------------------------------------------------------------------------------
      Use `Faker` class for giving you random email address
      
      >>>  import logging
      >>>  from kyaah import Faker
      
      >>>  formatter = "[+] [%(asctime)s] [%(levelname)s] %(message)s"
      >>>  logging.basicConfig(format = formatter)

      >>>  logger = logging.getLogger()
      >>>  logger.setLevel(logging.DEBUG)
      
      >>>  logger.info(Faker().faker())
      ------------------------------------------------------------------------------------------------------
"""

from . base import BaseMail
from . vault import Vault
from . utils import Tokens
from . utils import Faker




import json
import math
import random
import logging
import geocoder
import requests


"""
  NOTSET    ---  0
  DEBUG     ---  10
  INFO      ---  20
  WARNING   ---  30  (default)
  ERROR     ---  40
  CRITICAL  ---  50
"""


formatter = "[+] [%(asctime)s] [%(levelname)s] %(message)s"
logging.basicConfig(format = formatter)

# Creating an object
logger = logging.getLogger()
# Setting the threshold of logger to DEBUG
logger.setLevel(logging.DEBUG)


class Tokens:
  """The base class for OTP mail"""
  
  def mail_otp(self, _range=6):
    """send otp code"""
    digits = "0123456789"
    OTP = ""
    for _ in range(_range):
      OTP += digits[math.floor(random.random()*10)]
    return OTP



class Faker:
  """a class that give you random email address"""
  
  _source = ["https://www.guerrillamail.com/ajax.php?f=get_email_address&ip=", "&agent=Mozilla_foo_bar"]
  
  def __init__(self, url=_source, ip="me"):
    self.url = url
    self.ip = ip
    
    
  def what_ip(self):
    """find ip address of the requester"""
    g = geocoder.ip(self.ip).raw
    ip = g.get("ip")
    return ip
  
  
  @property
  def make_request(self):
    """this method make a request along side with an ip address from `what_ip` method of this class"""
    url_first = self.url[0]
    url_mid = self.url[1]
    url_end = self.what_ip()
    
    com_url = url_first + url_end + url_mid
    r = requests.get(com_url).text
    return r
    
    
  def faker(self) -> dict:
    """give the actual maill address generated from guerrillamail"""
    gue_raw = json.loads(self.make_request)
    _fake = gue_raw["email_addr"]
    return {"fake_email": _fake}





import os
from . import BaseMail


class Vault(BaseMail):
  """
  This class handle environment variable information
  
  use this class if you want to access your `mail address` and `password/app_password` from environment variable, all what you need is to put your `mail address environment variable` in place of your email address, likewise the password/app_password
  """
  
  @property
  def from_usr_addr(self):
    """get sender mail address from environment variable"""
    return os.environ.get(self.from_usr)
    
    
  @property
  def mail_passwd_env(self):
    """get sender mail password/app_password from environment variable"""
    return self.maill_passwd




from setuptools import setup
from setuptools import find_packages



setup(
  
  # name of the main package (kyaah)
  name='kyaah',
  version='0.1',
  
  description='Mail library that abstract away complexity of setting up mail server',
  long_description=open('README.md').read() + '\n\n' + open('CHANGELOG').read(),
  long_description_content_type="text/markdown",
  
  # The URL of your package's (project) home page e.g. github link
  url='https://github.com/usmanmusa1920/kyaah',
  
  # Author details
  author='Usman Musa',
  author_email='usmanmusa1920@gmail.com',
  
  # Choose your license, note the American spelling
  License='MIT',
  
  classifiers = [
    'Development Status :: 5 - Production/Stable',
    'Intended Audience :: Developers',
    'Natural Language :: English',
    'Operating System :: POSIX :: Linux',
    'License :: OSI Approved :: MIT License',
    'Programming Language :: Python',
    'Programming Language :: Python :: 3'
  ],
  
  # used when people are searching for a module, keywords separated with a space
  keywords='kyaah',
  
  # The list of packages(directories) for your library
  packages=find_packages(), # OR packages=['kyaah'] 
  
  # If your package is a single module, use this instead of 'packages':
  # py_modules=[''] # list of files (modules) that are not in any directory (at the root dir)
  # the libraries it depends on
  
  include_package_data = True, # include files listed in MANIFEST.in
  
  # List of other python modules which this module depends on.  For example RPi.GPIO
  install_requires=['geocoder==1.38.1']
)



import imghdr
import logging
import smtplib
from email.message import EmailMessage


formatter = "[+] [%(asctime)s] [%(levelname)s] %(message)s"
logging.basicConfig(format = formatter)

logger = logging.getLogger()
logger.setLevel(logging.DEBUG)



class BaseMail:
  """The base class for mail"""
  
  def __init__(
    self,
    from_usr = None,
    to_usr = None,
    subject = None,
    body = None,
    mail_passwd = None,
    server = None,
    ):
    
    self.from_usr = from_usr
    self.to_usr = to_usr
    self.subject = subject
    self.body = body
    self.mail_passwd = mail_passwd
    self.server = server
    
    
    while self.from_usr == None:
      raise UnboundLocalError(f"`from_usr` is required")
    while self.to_usr == None:
      raise UnboundLocalError(f"`to_usr` is required")
    
  @property
  def from_usr_addr(self):
    """the sender mail address"""
    return self.from_usr
  
  
  @property
  def mail_passwd_env(self):
    """the sender mail password"""
    return self.mail_passwd
  
  
  @property
  def mail_msg(self):
    """content of the mail (subject and message body)"""
    subj = self.subject
    body = self.body
    data = f"Subject: {subj}\n\n{body}"
    return data
  
  
  def check_required(self):
    """checking for required values"""
    
    while self.mail_passwd == None:
      raise UnboundLocalError(f"`mail_passwd` is required")
    while self.server == None:
      raise UnboundLocalError(f"`server` is required")
      
      
  def mail_head(self, msg):
    """contains the sender, receiver and subject"""
    # grab the subject from `mail_msg` method by spliting it,
    # and grab the first index which is the subject,
    # then we slice the first index from index `9` that mean we negate
    # `subj` variable of mail_msg method of this class
    # which is `Subject: `
    msg['Subject'] = self.mail_msg.split('\n')[0][9:]
    msg['From'] = self.from_usr_addr
    msg['To'] = self.to_usr
    
    
  def mail_open(self, msg, port):
    """this open the connection, and login to email address using email address provided together with password/app_password"""
    with smtplib.SMTP_SSL(self.server, port) as smtp:
      smtp.login(self.from_usr_addr, self.mail_passwd_env)
      smtp.send_message(msg)
      
      
  def local_mail(self, port=1025):
    """localhost mail"""
    with smtplib.SMTP("localhost", port) as smtp:
      smtp.sendmail(self.from_usr_addr, self.to_usr, self.mail_msg)
      
      
  def send_mail(self, port=25):
    """send mail with ssl security"""
    
    """recepients is the list of receivers email address that you will send emaill address to, if the recepients is not included when call the method it will only send it to the `self.to_usr` that you pass in as the argument when instanciating the `BaseMail` class"""
    
    self.check_required()
    
    if len(self.to_usr) > 1:
      """if there is recepients"""
      for receiver in self.to_usr:
        with smtplib.SMTP_SSL(self.server, port) as smtp:
          smtp.login(self.from_usr_addr, self.mail_passwd_env)
          smtp.sendmail(self.from_usr_addr, receiver, self.mail_msg)
    elif len(self.to_usr) == 1:
      with smtplib.SMTP_SSL(self.server, port) as smtp:
        smtp.login(self.from_usr_addr, self.mail_passwd_env)
        smtp.sendmail(self.from_usr_addr, self.to_usr, self.mail_msg)
      
      
  def mail_with_image(self, image, port=25):
    """send mail together with an image"""
    
    self.check_required()
    
    if len(self.to_usr) > 1:
      """if the list of to_usr is more than one"""
      
      for receiver in self.to_usr:
        msg = EmailMessage()
        msg['Subject'] = self.mail_msg.split('\n')[0][9:]
        msg['From'] = self.from_usr_addr
        msg['To'] = receiver
        msg.set_content(self.mail_msg.split('\n')[2])
        
        if type(image) == list:
          for my_img in image:
            with open(my_img, 'rb') as img_send:
              img_read = img_send.read()
              f_type = imghdr.what(img_send.name)
              f_name = img_send.name
            msg.add_attachment(img_read, maintype='image', subtype=f_type, filename=f_name)
          self.mail_open(msg, port)
        elif type(image) == str:
          with open(image, 'rb') as img_send:
            img = img_send.read()
            f_type = imghdr.what(img_send.name)
            f_name = img_send.name
          msg.add_attachment(img, maintype='image', subtype=f_type, filename=f_name)
          self.mail_open(msg, port)
        else:
          logger.error(f"only list or string can be pass as argument in {self.mail_with_image}")
    elif len(self.to_usr) == 1:
      msg = EmailMessage()
      self.mail_head(msg)
      msg.set_content(self.mail_msg.split('\n')[2])
      
      if type(image) == list:
        for my_img in image:
          with open(my_img, 'rb') as img_send:
            img_read = img_send.read()
            f_type = imghdr.what(img_send.name)
            f_name = img_send.name
          msg.add_attachment(img_read, maintype='image', subtype=f_type, filename=f_name)
        self.mail_open(msg, port)
      elif type(image) == str:
        with open(image, 'rb') as img_send:
          img = img_send.read()
          f_type = imghdr.what(img_send.name)
          f_name = img_send.name
        msg.add_attachment(img, maintype='image', subtype=f_type, filename=f_name)
        self.mail_open(msg, port)
      else:
        logger.error(f"only list or string can be pass as argument in {self.mail_with_image}")
      
      
  def mail_with_file(self, mail_files, port=25):
    """send mail together with a list of mail_files"""
    
    self.check_required()
    
    msg = EmailMessage()
    self.mail_head(msg)
    msg.set_content(self.mail_msg.split('\n')[2])
    
    for a_file in mail_files:
      with open(a_file, 'rb') as file_send:
        f_read = file_send.read()
        f_name = file_send.name
      msg.add_attachment(f_read, maintype='application', subtype='octet-stream', filename=f_name)
    self.mail_open(msg, port)
    
    
  def mail_with_page(self, file=None, port=25):
    """send mail with html page. You can pass a keyword args of `file='default'` which will send a default html page provided within this class method, that is for test
    
    But if you want to send yor own html page assign the `file` with a value of the absolute location of your html page"""
    
    self.check_required()
    
    msg = EmailMessage()
    self.mail_head(msg)
    msg.set_content(self.mail_msg.split('\n')[2])
    
    if file == 'default':
      dummy_page = f"""<!DOCTYPE html>
<html lang="en">
<head>
  <meta charset="UTF-8">
  <meta http-equiv="X-UA-Compatible" content="IE=edge">
  <meta name="viewport" content="width=device-width, initial-scale=1.0">
  <title>Document</title>
</head>
<body>
  <h1 style="color:SlateGray;">Hello world mail from {self.from_usr_addr}</h1>
  <p style="color:brown;">Lorem ipsum dolor sit amet consectetur adipisicing elit. Rerum voluptate ipsum voluptatum doloribus, incidunt totam doloremque quae quibusdam exercitationem sapiente vel veritatis consequuntur earum et. Quod nobis minus minima repellat.</p>
</body>
</html>
"""
      msg.add_alternative(dummy_page, subtype='html')
    elif file != None:
      with open(file) as f_html:
        html_f = f_html.read()
      msg.add_attachment(html_f, subtype='html')
    self.mail_open(msg, port)

